@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2008 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2010 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,6 +21,10 @@ import sys
 import os
 import unittest
 import email
+try:
+    from Mailman import __init__
+except ImportError:
+    import paths
 
 from Mailman.Bouncers.BouncerAPI import Stop
 
@@ -71,6 +75,9 @@ class BounceTest(unittest.TestCase):
         ('SimpleMatch', 'simple_31.txt', ['nmorel@actisce.fr']),
         ('SimpleMatch', 'simple_32.txt', ['teteyn@agence-forbin.com']),
         ('SimpleMatch', 'simple_33.txt', ['hmu@extralumin.com']),
+        ('SimpleMatch', 'simple_34.txt', ['roland@xxx.com']),
+        ('SimpleMatch', 'simple_36.txt', ['garyt@xxx.com']),
+        ('SimpleMatch', 'simple_37.txt', ['user@uci.edu']),
         ('SimpleMatch', 'bounce_02.txt', ['acinsp1@midsouth.rr.com']),
         ('SimpleMatch', 'bounce_03.txt', ['james@jeborall.demon.co.uk']),
         # SimpleWarning
@@ -78,6 +85,7 @@ class BounceTest(unittest.TestCase):
         ('SimpleWarning', 'simple_21.txt', Stop),
         ('SimpleWarning', 'simple_22.txt', Stop),
         ('SimpleWarning', 'simple_28.txt', Stop),
+        ('SimpleWarning', 'simple_35.txt', Stop),
         # GroupWise
         ('GroupWise', 'groupwise_01.txt', ['thoff@MAINEX1.ASU.EDU']),
         # This one really sucks 'cause it's text/html.  Just make sure it
@@ -125,6 +133,7 @@ class BounceTest(unittest.TestCase):
         ('Qmail', 'qmail_03.txt', ['crown@hbc.co.jp']),
         ('Qmail', 'qmail_04.txt', ['merotiia@tennisnsw.com.au']),
         ('Qmail', 'qmail_05.txt', ['ivokggrrdvc@caixaforte.freeservers.com']),
+        ('Qmail', 'qmail_06.txt', ['ntl@xxx.com']),
         # LLNL's custom Sendmail
         ('LLNL', 'llnl_01.txt', ['trotts1@llnl.gov']),
         # Netscape's server...
@@ -159,6 +168,7 @@ class BounceTest(unittest.TestCase):
                                    'turtle4jne@yahoo.com']),
         # sina.com appears to use their own weird SINAEMAIL MTA
         ('Sina', 'sina_01.txt', ['boboman76@sina.com', 'alan_t18@sina.com']),
+        ('AOL', 'aol_01.txt', ['screenname@aol.com']),
         # No address can be detected in these...
         # dumbass_01.txt - We love Microsoft. :(
         # Done
